/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/util.h#1 $
*/

/* Prevent the contents of this file from being included more than once */

#ifndef _UTIL_H_INCLUDED
#define _UTIL_H_INCLUDED

/*--------------------------------------------------------------------*/

#include <avs/port.h>
#ifndef malloc
#  include <malloc.h>
#endif

#ifdef AVS_MEM_DEBUG
/*
 * This will re-define malloc to be a call to MEMmalloc
 * with the __FILE__ and __LINE__ macros expanded
 */
#include <avs/mem_defs.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

   extern void db_crash_handler( int sig );

   extern void ERRout_of_memory( char *message );

#ifdef __cplusplus
}
#endif

/*--------------------------------------------------------------------*/
/* 
 * Allocate and initialize memory to zero.
 * All platforms now have calloc, so other options have been removed.
 * Requesting 0 or -ve bytes is bad news, caller should ensure N > 0
 */
#ifdef NDEBUG
#define ALLOCN(P,T,N,M) \
	{ size_t __UTIL_n = (N); \
	  while(1) { \
		(P) = (T *) calloc ( __UTIL_n, sizeof(T) ); \
		if ((P) != NULL) break; \
		ERRout_of_memory( M ); } \
	}	
#else
#define ALLOCN(P,T,N,M) \
	{ size_t __UTIL_n = (N); \
	  if (__UTIL_n < 1) { \
		fprintf( stderr, \
		 "Invalid memory allocation: %s line %d, %d bytes < 1\n", \
		 __FILE__, __LINE__, (int)(__UTIL_n*sizeof(T)) ); \
		 __UTIL_n = 1; } \
	  while(1) { \
		(P) = (T *) calloc ( __UTIL_n, sizeof(T) ); \
		if ((P) != NULL) break; \
		ERRout_of_memory( M ); } \
	}	
#endif

/*--------------------------------------------------------------------*/
/*
 * Allocate, but don't initialize the memory to zero.
 * The calling routine has the responsibility to 
 * initialize all the memory before use.
 * Requesting 0 or -ve bytes is bad news, caller should ensure N > 0
 */

#ifdef NDEBUG
#define UALLOCN(P,T,N,M) \
	{ size_t __UTIL_n_bytes = (N) * sizeof(T); \
	  while (1) { \
		(P) = (T *) malloc( __UTIL_n_bytes ); \
		if ((P) != NULL) break; \
		ERRout_of_memory( M ); } \
	}
#else
#define UALLOCN(P,T,N,M) \
	{ size_t __UTIL_n_bytes = (N) * sizeof(T); \
	  if (__UTIL_n_bytes < 1) { \
		fprintf (stderr, \
		   "Invalid memory allocation: %s line %d, %d bytes < 1\n", \
			 __FILE__, __LINE__, (int)__UTIL_n_bytes); \
		__UTIL_n_bytes = sizeof(int); } \
	  while (1) { \
		(P) = (T *) malloc( __UTIL_n_bytes ); \
		if ((P) != NULL) break; \
		ERRout_of_memory( M ); } \
	}
#endif

/*--------------------------------------------------------------------*/
/*
 * Reallocate, but don't zero any new memory. 
 * Requesting 0 or -ve bytes is bad news, caller should ensure N > 0
 */

#ifdef NDEBUG
#define REALLOC(P,T,N,M) \
	{ char   *__UTIL_old_ptr = (char*) (P); \
	  size_t  __UTIL_n_bytes = (N) * sizeof(T); \
	  while(1) { \
		(P) = (T *) realloc( __UTIL_old_ptr, __UTIL_n_bytes ); \
		if ((P) != NULL) break; \
		ERRout_of_memory( M ); } \
	}
#else
#define REALLOC(P,T,N,M) \
	{ char   *__UTIL_old_ptr = (char*) (P); \
	  size_t  __UTIL_n_bytes = (N) * sizeof(T); \
	  if (__UTIL_n_bytes < 1) { \
		fprintf (stderr, \
		   "Invalid memory reallocation: %s line %d, %d bytes < 1\n", \
			 __FILE__, __LINE__, (int)__UTIL_n_bytes); \
		__UTIL_n_bytes = sizeof(int); } \
	  while(1) { \
		(P) = (T *) realloc( __UTIL_old_ptr, __UTIL_n_bytes ); \
		if ((P) != NULL) break; \
		ERRout_of_memory( M ); } \
	}
#endif

/*--------------------------------------------------------------------*/
/* free(3) is non-ANSI compliant on SunOS 4.x */

#ifdef sol1
#define FREE(P)		free((char *) (P))
#else
#define FREE(P)		free(P)
#endif

/*--------------------------------------------------------------------*/
/* define yet another set of trivial math macros */

#define UTMIN(A,B)		((A) > (B) ? (B) : (A))
#define UTMAX(A,B)		((A) < (B) ? (B) : (A))
#define UTABS(A)		((A) < 0 ? -(A) : A)

/*--------------------------------------------------------------------*/
/* Function prototypes */

#ifdef __cplusplus
extern "C" {
#endif

void *UTILinitialize_wildcard PTARGS((
    char  *wildcard));

int UTILmatch_string_to_wildcard PTARGS((
    char  *string,
    void  *w));

void UTILfree_wildcard PTARGS((
    void  *w));

void
TMmerge_translation PTARGS((
    char  *tag,
    char  *translation));

char *
TMget_translation PTARGS((
    char  *tag,
    char  *default_value));

int
TMload_dictionary PTARGS((
    char  *filename, 
    char *build_path));

#ifdef __cplusplus
}       /* extern "C" */
#endif

/*--------------------------------------------------------------------*/

#endif /* _UTIL_H_INCLUDED */
